'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _logger = require('../lib/logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var InstagramSDK = (function () {
  function InstagramSDK() {
    var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var _ref$clientID = _ref.clientID;
    var clientID = _ref$clientID === undefined ? null : _ref$clientID;
    var _ref$clientSecret = _ref.clientSecret;
    var clientSecret = _ref$clientSecret === undefined ? null : _ref$clientSecret;
    var _ref$accessToken = _ref.accessToken;
    var accessToken = _ref$accessToken === undefined ? null : _ref$accessToken;

    _classCallCheck(this, InstagramSDK);

    this.instagramHost = 'api.instagram.com';
    this.apiPath = '/v1';

    if (!clientID && !clientSecret && !accessToken) {
      throw new Error('You must specify on of: `accessToken` or both `clientID` and `clientSecret`.');
    }

    this.clientID = clientID;
    this.clientSecret = clientSecret;
    this.accessToken = accessToken;

    if (!accessToken && clientID && clientSecret) {
      this.printLoginUrl();
    } else if (accessToken) {}
  }

  _createClass(InstagramSDK, [{
    key: 'printLoginUrl',
    value: function printLoginUrl() {
      console.log('https://' + this.instagramHost + '/oauth/authorize/?client_id=' + this.clientID + '&redirect_uri=http://poster.loc&response_type=code');
    }

    //////////////////////////
    //// Users
    //////////////////////////

  }, {
    key: 'getSelf',
    value: function getSelf() {
      return this._request().then(this._parseJSON);
    }
  }, {
    key: 'getSelfRecentMedia',
    value: function getSelfRecentMedia() {
      var _ref2 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref2$count = _ref2.count;
      var count = _ref2$count === undefined ? 10 : _ref2$count;
      var _ref2$min_id = _ref2.min_id;
      var min_id = _ref2$min_id === undefined ? undefined : _ref2$min_id;
      var _ref2$max_id = _ref2.max_id;
      var max_id = _ref2$max_id === undefined ? undefined : _ref2$max_id;

      return this._request({ path: '/users/self/media/recent', query: { count: count, min_id: min_id, max_id: max_id } }).then(this._parseJSON);
    }
  }, {
    key: 'getSelfRecentLikes',
    value: function getSelfRecentLikes() {
      var _ref3 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref3$count = _ref3.count;
      var count = _ref3$count === undefined ? 10 : _ref3$count;
      var _ref3$max_like_id = _ref3.max_like_id;
      var max_like_id = _ref3$max_like_id === undefined ? undefined : _ref3$max_like_id;

      return this._request({ path: '/users/self/media/liked', query: { count: count, max_like_id: max_like_id } }).then(this._parseJSON);
    }
  }, {
    key: 'getUser',
    value: function getUser(userID) {
      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      return this._request({ path: '/users/' + userID }).then(this._parseJSON);
    }
  }, {
    key: 'getUserRecentMedia',
    value: function getUserRecentMedia(userID) {
      var _ref4 = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var _ref4$count = _ref4.count;
      var count = _ref4$count === undefined ? 10 : _ref4$count;
      var _ref4$min_id = _ref4.min_id;
      var min_id = _ref4$min_id === undefined ? undefined : _ref4$min_id;
      var _ref4$max_id = _ref4.max_id;
      var max_id = _ref4$max_id === undefined ? undefined : _ref4$max_id;

      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      return this._request({ path: '/users/' + userID + '/media/recent', query: { count: count, min_id: min_id, max_id: max_id } }).then(this._parseJSON);
    }
  }, {
    key: 'usersSearch',
    value: function usersSearch() {
      var _ref5 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref5$q = _ref5.q;
      var q = _ref5$q === undefined ? '' : _ref5$q;
      var _ref5$count = _ref5.count;
      var count = _ref5$count === undefined ? 10 : _ref5$count;

      return this._request({ path: '/users/search', query: { q: q, count: count } }).then(this._parseJSON);
    }

    //////////////////////////
    //// Relationships
    //////////////////////////

  }, {
    key: 'getSelfFollows',
    value: function getSelfFollows() {
      return this._request({ path: '/users/self/follows' }).then(this._parseJSON);
    }
  }, {
    key: 'getSelfFollowedBy',
    value: function getSelfFollowedBy() {
      return this._request({ path: '/users/self/followed-by' }).then(this._parseJSON);
    }
  }, {
    key: 'getSelfRequestedBy',
    value: function getSelfRequestedBy() {
      return this._request({ path: '/users/self/requested-by' }).then(this._parseJSON);
    }
  }, {
    key: 'getUserRelationship',
    value: function getUserRelationship(userID) {
      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      return this._request({ path: '/users/' + userID + '/relationship' }).then(this._parseJSON);
    }
  }, {
    key: 'updateUserRelationship',
    value: function updateUserRelationship(userID, action) {
      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      if (!action) {
        throw new Error('Argument `action` is required.');
      }

      return this._request({ method: 'POST', path: '/users/' + userID + '/relationship', postData: { action: action } }).then(this._parseJSON);
    }

    //////////////////////////
    //// Media
    //////////////////////////

  }, {
    key: 'getMediaInfoById',
    value: function getMediaInfoById(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      return this._request({ path: '/media/' + mediaID }).then(this._parseJSON);
    }
  }, {
    key: 'getMediaInfoByShortCode',
    value: function getMediaInfoByShortCode(shortCode) {
      if (!shortCode) {
        throw new Error('Argument `shortCode` is required.');
      }

      return this._request({ path: '/media/shortcode/' + shortCode }).then(this._parseJSON);
    }
  }, {
    key: 'mediaSearch',
    value: function mediaSearch() {
      var _ref6 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref6$lat = _ref6.lat;
      var lat = _ref6$lat === undefined ? 0 : _ref6$lat;
      var _ref6$lng = _ref6.lng;
      var lng = _ref6$lng === undefined ? 0 : _ref6$lng;
      var _ref6$distance = _ref6.distance;
      var distance = _ref6$distance === undefined ? 10 : _ref6$distance;

      return this._request({ path: '/media/search', query: { lat: lat, lng: lng, distance: distance } }).then(this._parseJSON);
    }

    //////////////////////////
    //// Comments
    //////////////////////////

  }, {
    key: 'getCommentsForMedia',
    value: function getCommentsForMedia(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      return this._request({ path: '/media/' + mediaID + '/comments' }).then(this._parseJSON);
    }
  }, {
    key: 'addCommentForMedia',
    value: function addCommentForMedia(mediaID, text) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      if (!text) {
        throw new Error('Argument `text` is required.');
      }

      return this._request({ method: 'POST', path: '/media/' + mediaID + '/comments', postData: { text: text } }).then(this._parseJSON);
    }
  }, {
    key: 'removeCommentForMedia',
    value: function removeCommentForMedia(mediaID, commentId) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      if (!commentId) {
        throw new Error('Argument `commentId` is required.');
      }

      return this._request({ method: 'DELETE', path: '/media/' + mediaID + '/comments/' + commentId }).then(this._parseJSON);
    }

    //////////////////////////
    //// Likes
    //////////////////////////

  }, {
    key: 'getLikesForMedia',
    value: function getLikesForMedia(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      return this._request({ path: '/media/' + mediaID + '/likes' }).then(this._parseJSON);
    }
  }, {
    key: 'addLikeForMedia',
    value: function addLikeForMedia(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      return this._request({ method: 'POST', path: '/media/' + mediaID + '/likes' }).then(this._parseJSON);
    }
  }, {
    key: 'removeLikeForMedia',
    value: function removeLikeForMedia(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      return this._request({ method: 'DELETE', path: '/media/' + mediaID + '/likes' }).then(this._parseJSON);
    }

    //////////////////////////
    //// Tags
    //////////////////////////

  }, {
    key: 'getTagInfoByTagName',
    value: function getTagInfoByTagName(tagName) {
      if (!tagName) {
        throw new Error('Argument `tagName` is required.');
      }

      return this._request({ path: '/tags/' + tagName }).then(this._parseJSON);
    }
  }, {
    key: 'getRecentMediaForTagName',
    value: function getRecentMediaForTagName(tagName) {
      var _ref7 = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var _ref7$count = _ref7.count;
      var count = _ref7$count === undefined ? 10 : _ref7$count;
      var _ref7$min_tag_id = _ref7.min_tag_id;
      var min_tag_id = _ref7$min_tag_id === undefined ? undefined : _ref7$min_tag_id;
      var _ref7$max_tag_id = _ref7.max_tag_id;
      var max_tag_id = _ref7$max_tag_id === undefined ? undefined : _ref7$max_tag_id;

      if (!tagName) {
        throw new Error('Argument `tagName` is required.');
      }

      return this._request({ path: '/tags/' + tagName + '/media/recent', query: { count: count, min_tag_id: min_tag_id, max_tag_id: max_tag_id } }).then(this._parseJSON);
    }
  }, {
    key: 'tagsSearch',
    value: function tagsSearch() {
      var q = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

      return this._request({ path: '/tags/search', query: { q: q } }).then(this._parseJSON);
    }

    //////////////////////////
    //// Locations
    //////////////////////////

  }, {
    key: 'getLocationInfoByLocationId',
    value: function getLocationInfoByLocationId(locationId) {
      if (!locationId) {
        throw new Error('Argument `locationId` is required.');
      }

      return this._request({ path: '/locations/' + locationId }).then(this._parseJSON);
    }
  }, {
    key: 'getRecentMediaForLocationId',
    value: function getRecentMediaForLocationId(locationId) {
      var _ref8 = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var _ref8$min_tag_id = _ref8.min_tag_id;
      var min_tag_id = _ref8$min_tag_id === undefined ? undefined : _ref8$min_tag_id;
      var _ref8$max_tag_id = _ref8.max_tag_id;
      var max_tag_id = _ref8$max_tag_id === undefined ? undefined : _ref8$max_tag_id;

      if (!locationId) {
        throw new Error('Argument `locationId` is required.');
      }

      return this._request({ path: '/locations/' + locationId + '/media/recent', query: { min_tag_id: min_tag_id, max_tag_id: max_tag_id } }).then(this._parseJSON);
    }
  }, {
    key: 'locationsSearch',
    value: function locationsSearch() {
      var _ref9 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref9$distance = _ref9.distance;
      var distance = _ref9$distance === undefined ? 1000 : _ref9$distance;
      var _ref9$facebook_places = _ref9.facebook_places_id;
      var facebook_places_id = _ref9$facebook_places === undefined ? undefined : _ref9$facebook_places;
      var _ref9$foursquare_id = _ref9.foursquare_id;
      var foursquare_id = _ref9$foursquare_id === undefined ? undefined : _ref9$foursquare_id;
      var _ref9$lat = _ref9.lat;
      var lat = _ref9$lat === undefined ? undefined : _ref9$lat;
      var _ref9$lng = _ref9.lng;
      var lng = _ref9$lng === undefined ? undefined : _ref9$lng;
      var _ref9$foursquare_v2_i = _ref9.foursquare_v2_id;
      var foursquare_v2_id = _ref9$foursquare_v2_i === undefined ? undefined : _ref9$foursquare_v2_i;

      return this._request({
        path: '/locations/search',
        query: {
          distance: distance,
          facebook_places_id: facebook_places_id,
          foursquare_id: foursquare_id,
          lat: lat,
          lng: lng,
          foursquare_v2_id: foursquare_v2_id
        }
      }).then(this._parseJSON);
    }

    //////////////////////////
    //// Embedding
    //////////////////////////

  }, {
    key: 'getMediaJPGByShortCode',
    value: function getMediaJPGByShortCode(shortCode) {
      if (!shortCode) {
        throw new Error('Argument `shortCode` is required.');
      }

      return this._request({ path: '/p/' + shortCode + '/media' }).then(this._getHeaderValue('Location'));
    }
  }, {
    key: '_getHeaderValue',
    value: function _getHeaderValue(headerName) {
      return function () {
        var _ref10 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

        var res = _ref10.res;

        return _bluebird2.default.resolve(res.headers[headerName]);
      };
    }
  }, {
    key: '_parseJSON',
    value: function _parseJSON() {
      var _ref11 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var res = _ref11.res;
      var resData = _ref11.resData;

      console.log('_parseJSON called');
      var resJSON = {};

      if (res.statusCode == 204) {
        return _bluebird2.default.resolve({});
      }

      try {
        resJSON = JSON.parse(resData);
      } catch (err) {
        _logger2.default.debug('_parseJSON: invalid response:', resData);
        return _bluebird2.default.reject(new Error('Invalid JSON response:' + resData));
      }

      _logger2.default.debug('_parseJSON: response: %:2j', resJSON);
      return _bluebird2.default.resolve(resJSON);
    }
  }, {
    key: '_request',
    value: function _request() {
      var _this = this;

      var _ref12 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref12$method = _ref12.method;
      var method = _ref12$method === undefined ? 'GET' : _ref12$method;
      var _ref12$path = _ref12.path;
      var path = _ref12$path === undefined ? '/users/self' : _ref12$path;
      var postData = _ref12.postData;
      var _ref12$query = _ref12.query;
      var query = _ref12$query === undefined ? {} : _ref12$query;

      return new _bluebird2.default(function (resolve, reject) {
        query.access_token = _this.accessToken;
        var contentType = 'application/json',
            headers = {
          Accept: contentType
        },
            requestOptions = {
          hostname: _this.instagramHost,
          path: _this.apiPath + path + '?' + _querystring2.default.stringify(query),
          method: method
        },
            _postData = undefined;

        if (postData) {
          _postData = JSON.stringify(postData);
          headers['Content-Type'] = 'application/json';
          headers['Content-length'] = _postData.length;
          _logger2.default.debug('_request: request with postData: %:2j', _postData);
        }

        requestOptions.headers = headers;
        _logger2.default.debug('_request: request with params: %:2j', requestOptions);

        var request = _https2.default.request(requestOptions, function (res) {
          var resData = '';

          res.on('data', function (data) {
            resData += data;
          });
          res.on('end', function () {
            _logger2.default.debug('_request: response statusCode:', res.statusCode);
            resolve({ res: res, resData: resData });
          });
        }).on('error', reject);

        if (postData) {
          request.write(_postData);
        }

        request.end();
      });
    }
  }]);

  return InstagramSDK;
})();

exports.default = InstagramSDK;